// This source is subject to the Microsoft Public License.
// See http://www.microsoft.com/resources/sharedsource/licensingbasics/sharedsourcelicenses.mspx.
// All other rights reserved.

using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Resources;
using System.Threading;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Xml;

using Infragistics.WebUI.UltraWebNavigator;
using Infragistics.WebUI.UltraWebGrid;



namespace QuoteAdvantage
{
   /// <summary>
   /// Main window for QuoteAdvantage
   /// </summary>
   public class CustomizeQuote : System.Web.UI.Page
   {
      protected Quote objQuote = null;
      protected string m_strQuoteId = "";
      protected string strCategoryId = "";
      protected Engine eng = null;
      protected Organization objOrganization = null;
      protected ResourceManager m_mgrResource;
      protected int nCurrencyFormatCode;
      protected string strCurrencySymbol;
      protected string strParentProductCustomHeader1 = "";
      protected string strParentProductCustomHeader2 = "";
      protected System.Drawing.Color objCustomizeLinkColor = Color.Blue;
      protected XmlNode nodeCategoryList = null;
      protected string strOrganizationName;

      protected Infragistics.WebUI.UltraWebNavigator.UltraWebTree ctlTree;
      protected System.Web.UI.WebControls.Label lblProductCategories;
      protected System.Web.UI.WebControls.Label lblAvailableItemsInCategory;
      protected System.Web.UI.WebControls.Label lblCustomizableProducts;
      protected Infragistics.WebUI.UltraWebGrid.UltraWebGrid gridAvailableProducts2;
      protected System.Web.UI.WebControls.Button btnAddItem;
      protected System.Web.UI.WebControls.TextBox txtProductId;
      protected System.Web.UI.WebControls.Button btnRemoveItem;
      protected System.Web.UI.WebControls.ImageButton btnPrint;
      protected System.Web.UI.WebControls.ImageButton btnDiscount;
      protected Infragistics.WebUI.UltraWebGrid.UltraWebGrid gridQuoteItems2;
      protected HyperLink lnkSupport;

      protected Label lblTestResult;

      private void Page_Load(object sender, System.EventArgs e)
      {
         if (this.Trace.IsEnabled)
         {
            this.Trace.Write("Starting Page Load event.");
            this.Trace.Write("QueryString: " + Request.QueryString.ToString());
            this.Trace.Write("Form Variables: " + Request.Form.Count);
         }

         m_strQuoteId = Request.QueryString["id"];  // CRM 4.0

         if (m_strQuoteId == null)
         {
            m_strQuoteId = Request.QueryString["oId"];  // CRM 3.0
         }

         // TESTING
         if (m_strQuoteId == null)
         {
            m_strQuoteId = ConfigurationManager.AppSettings["TestQuoteId"];
         }

         if (this.Trace.IsEnabled)
         {
            this.Trace.Write("QuoteId=" + m_strQuoteId);
         }

         if (Request.QueryString["CategoryId"] != null)
         {
            strCategoryId = Request.QueryString["CategoryId"];
         }

         if (Request.QueryString["orgname"] != null)
         {
            strOrganizationName = Request.QueryString["orgname"];
         }

         // Set the culture and UI culture to the browser's accept language
         Thread.CurrentThread.CurrentCulture = CultureInfo.CreateSpecificCulture(Request.UserLanguages[0]);
         Thread.CurrentThread.CurrentUICulture = new CultureInfo(Request.UserLanguages[0]);

         m_mgrResource = new ResourceManager("QuoteAdvantage.strings", typeof(CustomizeQuote).Assembly);

         if (this.Trace.IsEnabled)
            this.Trace.Write("Initializing Engine");
         eng = new Engine(this.Trace, m_mgrResource);

         objQuote = eng.LoadQuote(new Guid(m_strQuoteId));
         objOrganization = eng.LoadOrganization();
         strCurrencySymbol = objOrganization.CurrencySymbol;
         nCurrencyFormatCode = objOrganization.CurrencyFormatCode;

         // Load link color for the customize links in the quote items grid
         if (ConfigurationManager.AppSettings["CustomizeLinkColor"] != null)
         {
            if (ConfigurationManager.AppSettings["CustomizeLinkColor"] != "")
            {
               objCustomizeLinkColor = Color.FromName(ConfigurationManager.AppSettings["CustomizeLinkColor"]);
            }
         }

         LoadCategoryList();

         if (!this.IsPostBack)
         {
            if (this.Trace.IsEnabled)
               this.Trace.Write("Initializing Page...");

            lnkSupport.NavigateUrl = "Support.aspx?orgname=" + strOrganizationName;
            LoadResources();
            LoadProductCategoryList(ctlTree);
            SelectCategory(strCategoryId);
            RefreshAvailableProducts();
            RefreshQuoteItems();

            btnPrint.Attributes.Add("onclick", "return onPrintClicked()");
            if (this.Trace.IsEnabled)
               this.Trace.Write("Page Complete.");
         }
      }

      #region Web Form Designer generated code
      override protected void OnInit(EventArgs e)
      {
         //
         // CODEGEN: This call is required by the ASP.NET Web Form Designer.
         //
         InitializeComponent();
         base.OnInit(e);
      }

      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent()
      {
         this.ctlTree.NodeClicked += new Infragistics.WebUI.UltraWebNavigator.NodeClickedEventHandler(this.ctlTree_NodeClicked);
         this.btnDiscount.Click += new System.Web.UI.ImageClickEventHandler(this.btnDiscount_Click);
         this.gridAvailableProducts2.InitializeRow += new Infragistics.WebUI.UltraWebGrid.InitializeRowEventHandler(this.gridAvailableProducts2_InitializeRow);
         this.gridQuoteItems2.InitializeRow += new Infragistics.WebUI.UltraWebGrid.InitializeRowEventHandler(this.gridQuoteItems2_InitializeRow);
         this.gridQuoteItems2.UpdateCell += new Infragistics.WebUI.UltraWebGrid.UpdateCellEventHandler(this.gridQuoteItems2_UpdateCell);
         this.btnRemoveItem.Click += new System.EventHandler(this.btnRemoveItem_Click);
         this.btnAddItem.Click += new System.EventHandler(this.btnAddItem_Click);
         this.Load += new System.EventHandler(this.Page_Load);

      }
      #endregion

      private void LoadResources()
      {
         lblProductCategories.Text = m_mgrResource.GetString("lblProductCategories");
         lblAvailableItemsInCategory.Text = m_mgrResource.GetString("lblAvailableItemsInCategory");
         lblCustomizableProducts.Text = m_mgrResource.GetString("lblCustomizableProducts");
         btnPrint.ToolTip = m_mgrResource.GetString("cmdGenerateProposal");

         gridAvailableProducts2.Columns[1].Header.Caption = m_mgrResource.GetString("hdrName");
         gridAvailableProducts2.Columns[2].Header.Caption = m_mgrResource.GetString("hdrProductId");
         gridAvailableProducts2.Columns[3].Header.Caption = m_mgrResource.GetString("hdrStandardCost");
         gridAvailableProducts2.Columns[4].Header.Caption = m_mgrResource.GetString("hdrListPrice");
         gridAvailableProducts2.Columns[5].Header.Caption = m_mgrResource.GetString("hdrPrice");

         gridQuoteItems2.Columns[0].Header.Caption = m_mgrResource.GetString("hdrName");
         gridQuoteItems2.Columns[2].Header.Caption = m_mgrResource.GetString("hdrBasePrice");
         gridQuoteItems2.Columns[3].Header.Caption = m_mgrResource.GetString("hdrQty");
         gridQuoteItems2.Columns[4].Header.Caption = m_mgrResource.GetString("hdrMultiplier");
         gridQuoteItems2.Columns[5].Header.Caption = m_mgrResource.GetString("hdrDiscountPercent");
         gridQuoteItems2.Columns[6].Header.Caption = m_mgrResource.GetString("hdrMarkupPercent");
         gridQuoteItems2.Columns[8].Header.Caption = m_mgrResource.GetString("hdrCustomizedPrice");

         // Hide/Show optional columns in the available product list
         gridAvailableProducts2.Columns[3].Hidden = true;
         gridAvailableProducts2.Columns[4].Hidden = true;

         if (ConfigurationManager.AppSettings["ShowStandardCostColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowStandardCostColumn"]);
            gridAvailableProducts2.Columns[3].Hidden = !bVisible;
         }

         if (ConfigurationManager.AppSettings["ShowListPriceColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowListPriceColumn"]);
            gridAvailableProducts2.Columns[4].Hidden = !bVisible;
         }

         // Load the names of any custom fields that need to be included
         // Quote Item columns are:
         //    0: Name
         //    1: Custom 1
         //    2: Base Price
         //    3: Qty
         //    4: Multiplier
         //    5: Discount %
         //    6: Markup %
         //    7: Custom 2
         //    8: Customized Price
         //    9: Remove
         if (ConfigurationManager.AppSettings["ParentProductCustomHeader1"] != null)
         {
            gridQuoteItems2.Columns[1].Header.Caption = ConfigurationManager.AppSettings["ParentProductCustomHeader1"];
            gridQuoteItems2.Columns[1].Hidden = false;
         }

         if (ConfigurationManager.AppSettings["ParentProductCustomHeader2"] != null)
         {
            gridQuoteItems2.Columns[7].Header.Caption = ConfigurationManager.AppSettings["ParentProductCustomHeader2"];
            gridQuoteItems2.Columns[7].Hidden = false;
         }

         if (ConfigurationManager.AppSettings["ShowMultiplierColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowMultiplierColumn"]);
            gridQuoteItems2.Columns[4].Hidden = !bVisible;
         }

         if (ConfigurationManager.AppSettings["ShowQtyColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowQtyColumn"]);
            gridQuoteItems2.Columns[3].Hidden = !bVisible;
         }

         if (ConfigurationManager.AppSettings["ShowPercentDiscountColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowPercentDiscountColumn"]);
            gridQuoteItems2.Columns[5].Hidden = !bVisible;
         }

         if (ConfigurationManager.AppSettings["ShowPercentMarkupColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowPercentMarkupColumn"]);
            gridQuoteItems2.Columns[6].Hidden = !bVisible;
         }

         if (ConfigurationManager.AppSettings["ShowCustomizedPriceColumn"] != null)
         {
            bool bVisible = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowCustomizedPriceColumn"]);
            gridQuoteItems2.Columns[8].Hidden = !bVisible;
         }

         btnDiscount.Visible = false;
         if (ConfigurationManager.AppSettings["CustomDiscountUrl"] != null)
         {
            if (ConfigurationManager.AppSettings["CustomDiscountUrl"] != "")
            {
               btnDiscount.Visible = true;
            }
         }

         if (objQuote.IsReadOnly)
         {
            gridQuoteItems2.Columns[3].AllowUpdate = AllowUpdate.No;
            gridQuoteItems2.Columns[4].AllowUpdate = AllowUpdate.No;
            gridQuoteItems2.Columns[5].AllowUpdate = AllowUpdate.No;
            gridQuoteItems2.Columns[6].AllowUpdate = AllowUpdate.No;
         }
      }

      private void AddItem(string strProductID, string strCategoryId)
      {
         Product prod = eng.LoadProduct(new Guid(strProductID), objQuote.PriceLevelId);

         XmlNode nodeProduct = eng.GetProductNode(nodeCategoryList, strCategoryId, prod.ProductNumber);

         if (nodeProduct.Attributes["allowDiscounting"] != null)
         {
            prod.AllowDiscounting = Convert.ToBoolean(nodeProduct.Attributes["allowDiscounting"].Value);
         }

         prod.CategoryId = strCategoryId;
         QuoteDetail objNewItem = eng.AddProduct(objQuote, prod);
         eng.AutoAddChildProducts(objQuote, objNewItem, objOrganization);

         // Reload the line items so that discounts are properly applied
         eng.LoadLineItems(objQuote);

         // Refresh the list of selected line items
         RefreshQuoteItems();
      }

      private void ConfigureItem(string strProductID)
      {
         Response.Redirect("CustomizeItem.aspx?orgname=" + strOrganizationName + "&QuoteId=" + m_strQuoteId + "&QuoteDetailId=" + strProductID);
      }

      private void RemoveItem(string strQuoteDetailID)
      {
         if (Trace.IsEnabled)
            Trace.Write("RemoveItem", "QuoteDetailId=" + strQuoteDetailID);

         QuoteDetail objItem = objQuote.FindQuoteDetail(new Guid(strQuoteDetailID));

         eng.RemoveParentItem(objItem);
         RefreshQuoteItems();

         if (Trace.IsEnabled)
            Trace.Write("RemoveItem", "Complete.");
      }

      public void LoadCategoryList()
      {
         nodeCategoryList = eng.LoadProductCategoryList();

         if (nodeCategoryList == null)
         {
            throw new ApplicationException("The <CategoryList> node was not found in the CategoryList.xml file.");
         }
      }

      public void LoadProductCategoryList(Infragistics.WebUI.UltraWebNavigator.UltraWebTree treeControl)
      {
         eng.DebugMessage("CustomizeQuote.LoadProductCategoryList() 1");

         Node nodeRoot = new Node();

         LoadChildItems(nodeRoot, nodeCategoryList);
         treeControl.Nodes.Clear();

         foreach (Node item in nodeRoot.Nodes)
         {
            treeControl.Nodes.Add(item);
         }

         eng.DebugMessage("CustomizeQuote.LoadProductCategoryList() 2");
      }

      private void LoadChildItems(Node treeParentNode, XmlNode xmlParentNode)
      {
         string strNodeName = "";

         try
         {
            foreach (XmlNode nodeChild in xmlParentNode.ChildNodes)
            {
               if (nodeChild.Name == "Category")
               {
                  strNodeName = nodeChild.Attributes["name"].Value;
                  Node nodeItem = new Node(strNodeName, null);

                  XmlAttribute attrib = nodeChild.Attributes["categoryId"];

                  // categoryId attribute is optional
                  if (attrib == null)
                     nodeItem.Tag = "";
                  else
                     nodeItem.Tag = attrib.Value;

                  LoadChildItems(nodeItem, nodeChild);
                  treeParentNode.Nodes.Add(nodeItem);
               }
            }
         }
         catch (Exception ex)
         {
            string strMessage = "There was an error processing the category tree item named: " + strNodeName;
            throw new ApplicationException(strMessage, ex);
         }
      }

      private void SelectCategory(string strTargetCategoryId)
      {
         if (ctlTree.Nodes.Count > 0)
         {
            ctlTree.SelectedNode = ctlTree.Nodes[0];

            if (strTargetCategoryId != "")
            {
               Node objCategoryNode = ctlTree.Find(strTargetCategoryId);

               if (objCategoryNode != null)
               {
                  ctlTree.SelectedNode = objCategoryNode;
               }
            }
         }
      }

      private void ctlTree_NodeClicked(object sender, Infragistics.WebUI.UltraWebNavigator.WebTreeNodeEventArgs e)
      {
         if (this.Trace.IsEnabled)
            this.Trace.Write("Tree node clicked. Item = " + e.Node.Tag.ToString());

         RefreshAvailableProducts();
      }

      private void RefreshAvailableProducts()
      {
         string strCategoryId = ctlTree.SelectedNode.Tag.ToString();

         lblAvailableItemsInCategory.Text = m_mgrResource.GetString("lblAvailableItemsInCategory");
         lblAvailableItemsInCategory.Text += " " + ctlTree.SelectedNode.Text;

         if (strCategoryId == "")
         {
            throw new ProductConfigException("There is an error in the CategoryList.xml file. A categoryId value was not found for this category:" + ctlTree.SelectedNode.Text);
         }
         else
         {
            ArrayList objProductList = eng.LoadCategoryItems(strCategoryId, objQuote.PriceLevelId);

            gridAvailableProducts2.DataSource = objProductList;
            gridAvailableProducts2.DataBind();
         }
      }

      private void RefreshQuoteItems()
      {
         gridQuoteItems2.DataSource = objQuote.GetConfigurableItems();
         gridQuoteItems2.DataBind();
      }

      protected string CalculateItemPrice(object val)
      {
         QuoteDetail objItem = (QuoteDetail)val;
         decimal nPrice = objItem.CalculatePrice();

         string strResult = Product.FormatCurrency(nPrice, nCurrencyFormatCode, strCurrencySymbol);

         return strResult;
      }

      private void gridAvailableProducts_ItemCreated(object sender, System.Web.UI.WebControls.DataGridItemEventArgs e)
      {
         ApplyGridHeaderStyle(e);
      }

      private void ApplyGridHeaderStyle(System.Web.UI.WebControls.DataGridItemEventArgs e)
      {
         if (e.Item.ItemType == ListItemType.Header)
         {
            foreach (TableCell cell in e.Item.Cells)
            {
               cell.BorderWidth = Unit.Pixel(1);
               cell.BorderStyle = BorderStyle.Solid;
               cell.BorderColor = Color.Gray;
               cell.Style.Add("border-left-color", "White");
               cell.Style.Add("border-left-width", "2px");
               cell.Style.Add("border-left-style", "Outset");
               cell.Style.Add("border-top-color", "LightGrey");
               cell.Style.Add("border-bottom-color", "DarkGray");
               cell.Style.Add("border-bottom-width", "2px");
               cell.Style.Add("border-top-style", "Outset");
               cell.Style.Add("border-bottom-style", "Solid");
               cell.Style.Add("padding-left", "3px");
            }
         }
      }

      // Quote Item columns are:
      //    0: Name
      //    1: Custom 1
      //    2: Base Price
      //    3: Qty
      //    4: Multiplier
      //    5: Discount %
      //    6: Markup %
      //    7: Custom 2
      //    8: Customized Price
      //    9: Remove
      private void gridQuoteItems2_InitializeRow(object sender, Infragistics.WebUI.UltraWebGrid.RowEventArgs e)
      {
         QuoteDetail objLineItem = (QuoteDetail)e.Data;
         string strRuleFileName = eng.GetRuleFileName(objLineItem);
         string strConfigureUrl = "javascript:customizeItem(\"CustomizeItem.aspx?orgname=" + strOrganizationName + "&QuoteId=" + m_strQuoteId + "&QuoteDetailId=" + objLineItem.QuoteDetailId + "\")";
         string strRemoveUrl = "javascript:removeItem(\"" + objLineItem.QuoteDetailId + "\");";
         string strRemoveCommand = m_mgrResource.GetString("cmdRemove");

         if (objQuote.IsReadOnly)
         {
            strRemoveUrl = "javascript:alert('This quote is read-only.')";
         }

         e.Row.Tag = objLineItem.QuoteDetailId.ToString();

         e.Row.Cells[0].Text = objLineItem.Name;
         e.Row.Cells[0].Title = objLineItem.Name;

         if (strRuleFileName != "")
         {
            e.Row.Cells[0].TargetURL = strConfigureUrl;
            e.Row.Cells[0].Style.ForeColor = objCustomizeLinkColor;
         }

         e.Row.Cells[2].Text = Product.FormatCurrency(objLineItem.PricePerUnit, nCurrencyFormatCode, strCurrencySymbol);

         decimal nMultiplier = objLineItem.ConvertFixedDiscountToPercentage();
         e.Row.Cells[4].Text = nMultiplier.ToString();

         e.Row.Cells[8].Text = CalculateItemPrice(objLineItem);

         e.Row.Cells[9].Text = strRemoveCommand;
         e.Row.Cells[9].TargetURL = strRemoveUrl;
      }

      private void gridQuoteItems2_UpdateCell(object sender, Infragistics.WebUI.UltraWebGrid.CellEventArgs e)
      {
         string strQuoteDetailId = e.Cell.Row.Tag.ToString();

         // Reload the line items so that discounts are properly applied
         eng.LoadLineItems(objQuote);
         QuoteDetail objLineItem = objQuote.FindQuoteDetail(new Guid(strQuoteDetailId));

         try
         {
            switch (e.Cell.Column.Index)
            {
               case 3:
                  // Quantity
                  int nNewQty = Convert.ToInt32(e.Cell.Text);

                  eng.ChangeParentItemQty(objQuote, strQuoteDetailId, nNewQty);

                  decimal nPercentageDiscount = Convert.ToDecimal(e.Cell.Row.Cells[4].Text);
                  objLineItem.ApplyPercentageMultiplier(nPercentageDiscount);

                  // Refresh the list of selected line items
                  RefreshQuoteItems();
                  break;

               case 4:
                  // Multiplier
                  decimal nNewMultiplier = Convert.ToDecimal(e.Cell.Text);

                  objLineItem.ApplyPercentageMultiplier(nNewMultiplier);

                  eng.UpdateQuoteDetail(objLineItem);

                  // Refresh the list of selected line items
                  RefreshQuoteItems();
                  break;

               case 5:
                  // Percent Discount
                  decimal nNewDiscount = Convert.ToDecimal(e.Cell.Text);

                  objLineItem.PercentDiscount = nNewDiscount;
                  objLineItem.RefreshMultiplier();

                  eng.UpdateQuoteDetail(objLineItem);
                  eng.UpdateQuote(objQuote);

                  // Refresh the list of selected line items
                  RefreshQuoteItems();
                  break;

               case 6:
                  // Percent Markup
                  decimal nNewMarkup = Convert.ToDecimal(e.Cell.Text);

                  objLineItem.PercentMarkup = nNewMarkup;
                  objLineItem.RefreshMultiplier();

                  eng.UpdateQuoteDetail(objLineItem);
                  eng.UpdateQuote(objQuote);

                  // Refresh the list of selected line items
                  RefreshQuoteItems();
                  break;

            }
         }
         catch (Exception ex)
         {
            Trace.Write("gridQuoteItems2_UpdateCell", "Error" + ex.Message);
         }
      }

      private void gridAvailableProducts2_InitializeRow(object sender, Infragistics.WebUI.UltraWebGrid.RowEventArgs e)
      {
         Product objProduct = (Product)e.Data;
         string strAddUrl = "javascript:addItem(\"" + objProduct.ProductId + "\");";
         string strAddCommand = m_mgrResource.GetString("cmdAdd");

         if (objQuote.IsReadOnly)
         {
            strAddUrl = "javascript:alert('This quote is read-only.')";
         }

         e.Row.Cells[0].Text = strAddCommand;
         e.Row.Cells[0].TargetURL = strAddUrl;
         e.Row.Cells[3].Text = Product.FormatCurrency(objProduct.StandardCost, nCurrencyFormatCode, strCurrencySymbol);
         e.Row.Cells[4].Text = Product.FormatCurrency(objProduct.ListPrice, nCurrencyFormatCode, strCurrencySymbol);
         e.Row.Cells[5].Text = Product.FormatCurrency(objProduct.Price, nCurrencyFormatCode, strCurrencySymbol);
      }

      private void btnAddItem_Click(object sender, System.EventArgs e)
      {
         string strProductId = txtProductId.Text;
         string strCategoryId = ctlTree.SelectedNode.Tag.ToString();

         AddItem(strProductId, strCategoryId);
      }

      private void btnRemoveItem_Click(object sender, System.EventArgs e)
      {
         string strQuoteDetailId = txtProductId.Text;
         RemoveItem(strQuoteDetailId);
      }

      private void btnDiscount_Click(object sender, System.Web.UI.ImageClickEventArgs e)
      {
         string strTargetUrl = ConfigurationManager.AppSettings["CustomDiscountUrl"];
         Response.Redirect(strTargetUrl + "?QuoteId=" + m_strQuoteId);
      }

   }  // end class
}
